"use strict";
// Copyright (c) 2020-2021 Brandon Lehmann
//
// Please see the included LICENSE file for more information.
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getDatabaseFromEnvironment = exports.createRedisCluster = exports.createRedisClient = exports.prepareCreateTable = exports.IDatabase = exports.Interfaces = exports.Redis = exports.Postgres = exports.MySQL = void 0;
const MySQL_1 = require("./MySQL");
exports.MySQL = MySQL_1.default;
const Postgres_1 = require("./Postgres");
exports.Postgres = Postgres_1.default;
const Redis = require("redis");
exports.Redis = Redis;
const redis_1 = require("redis");
Object.defineProperty(exports, "createRedisClient", { enumerable: true, get: function () { return redis_1.createClient; } });
Object.defineProperty(exports, "createRedisCluster", { enumerable: true, get: function () { return redis_1.createCluster; } });
const Types_1 = require("./Types");
Object.defineProperty(exports, "IDatabase", { enumerable: true, get: function () { return Types_1.IDatabase; } });
Object.defineProperty(exports, "Interfaces", { enumerable: true, get: function () { return Types_1.Interfaces; } });
Object.defineProperty(exports, "prepareCreateTable", { enumerable: true, get: function () { return Types_1.prepareCreateTable; } });
const dotenv_1 = require("dotenv");
var DatabaseType = Types_1.Interfaces.DBType;
var DatabasePort = Types_1.Interfaces.DBPort;
(0, dotenv_1.config)();
const getDatabaseFromEnvironment = (errorLog) => __awaiter(void 0, void 0, void 0, function* () {
    const create = (Type, ...args) => {
        const database = new Type(...args);
        if (errorLog) {
            database.on('error', (error) => errorLog(error));
        }
        return database;
    };
    const host = process.env.DB_HOST || 'localhost';
    let port = parseInt(process.env.DB_PORT || '0');
    const user = process.env.DB_USER || undefined;
    const pass = process.env.DB_PASSWORD || undefined;
    const db = process.env.DB_NAME || undefined;
    const dbFilename = process.env.DB_FILENAME || undefined;
    const dbType = (process.env.DB_USE_MYSQL)
        ? DatabaseType.MYSQL
        : (process.env.DB_USE_POSTGRES)
            ? DatabaseType.POSTGRES
            : (process.env.DB_USE_REDIS)
                ? DatabaseType.REDIS
                : DatabaseType.SQLITE;
    if (port === 0) {
        switch (dbType) {
            case Types_1.Interfaces.DBType.MYSQL:
                port = DatabasePort.MYSQL;
                break;
            case Types_1.Interfaces.DBType.POSTGRES:
                port = DatabasePort.POSTRGRES;
                break;
            case Types_1.Interfaces.DBType.REDIS:
                port = DatabasePort.REDIS;
                break;
            default:
                break;
        }
    }
    if (dbType === DatabaseType.MYSQL || dbType === DatabaseType.POSTGRES) {
        if (!host || !user || !pass || !db) {
            return undefined;
        }
    }
    else if (dbType === DatabaseType.SQLITE) {
        if (!dbFilename) {
            return undefined;
        }
    }
    switch (dbType) {
        case Types_1.Interfaces.DBType.MYSQL:
            return create(MySQL_1.default, host, port, user, pass, db);
        case Types_1.Interfaces.DBType.POSTGRES:
            return create(Postgres_1.default, host, port, user, pass, db);
        default: // redis has a totally different set of parameters
            return undefined;
    }
});
exports.getDatabaseFromEnvironment = getDatabaseFromEnvironment;
const DBAbstraction = {
    MySQL: MySQL_1.default,
    Postgres: Postgres_1.default,
    Redis,
    Interfaces: Types_1.Interfaces,
    IDatabase: Types_1.IDatabase,
    prepareCreateTable: Types_1.prepareCreateTable,
    createRedisClient: redis_1.createClient,
    createRedisCluster: redis_1.createCluster,
    getDatabaseFromEnvironment
};
exports.default = DBAbstraction;
